/* 
    Copyright (C) 2004  Mika Raento - Renaud Petit

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    email: mraento@cs.helsinki.fi - petit@cs.helsinki.fi 
*/


#if !defined(TRANSFER_H_INCLUDED)

#define TRANSFER_H_INCLUDED

#include <e32base.h>
#include <f32file.h>
#include <e32std.h>
#include <sendui.h>
#include <AknQueryDialog.h>

#include "file_output_base.h"
#include "status_notif.h"
#include "ftp.h"
#include "http.h"
#include "app_context.h"
#include "log_comm.h"
#include "xmlbuf.h"

class MUploadCallBack {
public:
	virtual void Back(bool Upload, bool DeleteFromPhone, CXmlBuf* Packet) = 0;
};

class MUploadPrompt {
public:
	virtual void Prompt(const TDesC& FileName, MUploadCallBack* CallBack) = 0;
};

class TDummyPrompt : public MUploadPrompt{
public:
	TDummyPrompt(bool Delete=true);
	virtual void Prompt(const TDesC& FileName, MUploadCallBack* CallBack);
private:
	bool	iDelete;
};

class CTransferBase: public MContextBase, public CBase, public MUploadCallBack, public MTimeOut
{
public:
	virtual ~CTransferBase();
	
	void add_filesL(const TDesC& file, bool leave_last);
private:
	virtual void Back(bool Upload, bool DeleteFromPhone,
		CXmlBuf* Packet);
	void DoBackL(bool Upload, bool DeleteFromPhone,
		CXmlBuf* Packet);
protected:
	void GetFiles(TTime AfterTime=TTime(0));
	void FileStep();
	virtual void GotFiles() = 0;
	virtual void expired();

	CDesC16ArrayFlat* file_names, *list_file_names; bool listing_files; bool again;
	int	file_index;

	CTransferBase(MApp_context& Context, MUploadPrompt& Prompt);
	void ConstructL(i_status_notif* callback, const TDesC& dir1, const TDesC& dir2);
	CDesC16ArrayFlat* send_dirs;
	CArrayFixFlat<bool>* send_leave;
	TFileName dir_prefix1, dir_prefix2;
	i_status_notif* cb;
	MUploadPrompt&	iPrompt;
	HBufC8*		iPacket8;
	CTimeOut*	iCallBack; bool in_call;

	TInt		send_dir_i, dir_i, dir_count;
	CDir		*dir;
	TTime		iAfterTime;
	TFileName	filen;
	TFileName	dirname; bool leave;
	TParse		p;
};

class CSendUITransfer: public CTransferBase {
public:
	static CSendUITransfer* NewL(MApp_context& Context, 
		i_status_notif* callback, TInt cmdid, const TDesC& dir1, const TDesC& dir2);
	void DisplayMenuL(CEikMenuPane& aMenuPane);
	void DisplaySendMenuL(CEikMenuPane& aMenuPane, TInt pos);
	bool transfer_files(TInt cmdid); // true when starting to write, false if already writing
	~CSendUITransfer();
private:
	virtual void GotFiles();

	bool in_progress;
	CSendUITransfer(MApp_context& Context);
	void ConstructL(i_status_notif* callback, TInt cmdid, const TDesC& dir1, const TDesC& dir2);
	CSendAppUi* sendui;
	TDummyPrompt	iDummyPrompt;
	TInt	iCmdId;
};

class CFtpTransfer: public CTransferBase, public MSocketObserver
{
public:
	enum TConnType { WAP, FTP, HTTP };
	static CFtpTransfer* NewL(MApp_context& Context, 
		i_status_notif* callback, const TDesC& dir1, const TDesC& dir2,
		TConnType ConnType, MUploadPrompt& Prompt);

	void TransferFiles(const TDesC& Meta=KNullDesC, TTime AfterTime=TTime(0));
	virtual ~CFtpTransfer();
private:
	void ConstructL(i_status_notif* callback, const TDesC& dir1, const TDesC& dir2,
		TConnType ConnType);
	CFtpTransfer(MApp_context& Context, MUploadPrompt& Prompt);
	virtual void GotFiles();
	void SendFile(); void GetFileInfoL(HBufC8*& this_packet, TDes& datetime);

	virtual void success(CBase* source);
	virtual void error(CBase* source, TInt code, const TDesC& reason);
	virtual void info(CBase* source, const TDesC& msg);
	bool GetImsi(TDes& aImsi);

	enum state { IDLE, CONNECTING, MKDING, CWDING, SENDING_FILE, CLOSING };
	state current_state;

	CHttp*	http;
	TConnType	iConnType;
	MAdvGsmPhoneInformation::TSubscriberId iImsi;
	TBuf<200>	iMeta;
	HBufC8*		iFtpPacket8;
	bool		iDeleteThisFile;
};

class CPeriodicTransfer: public MContextBase, public i_status_notif, public MTimeOut, public CBase
{
public:
	static CPeriodicTransfer* NewL(MApp_context& Context, int hours, MSocketObserver* callback);
	~CPeriodicTransfer();
	void Transfer(bool MakeCommLog=true);
private:
	CPeriodicTransfer(MApp_context& Context, int hours, MSocketObserver* callback);
	void ConstructL();

	enum state { IDLE, GETTING_COMM_LOG, FTPING };
	state current_state;
	void expired();
	virtual void finished();
	virtual void error(const TDesC& descr);
	virtual void status_change(const TDesC& status);

	CTimeOut	*iTimer;
	int		iHours, iCount;
	MSocketObserver	*iCb;
	CFtpTransfer	*iFtp;
	Clog_comm*	iCommLog;
	TTime		iCommLogFrom;
	TDummyPrompt	iDummyPrompt;
};


#endif
