/* 
    Copyright (C) 2004  Mika Raento - Renaud Petit

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    email: mraento@cs.helsinki.fi - petit@cs.helsinki.fi 
*/


/*
* ============================================================================
*  Name     : CSettingsView from SettingsView.h
*  Part of  : Settings
*  Created  : 13.1.2004 by 
*  Implementation notes:
*     Initial content was generated by Series 60 AppWizard.
*  Version  :
*  Copyright: 
* ============================================================================
*/

// INCLUDE FILES
#include  <aknviewappui.h>
#include  <avkon.hrh>
#include  <context_log.rsg>
#include  "SettingsView.h"
#include <aknsettingitemlist.h> 
#include <cdbcols.h>
#include <commdb.h>
#include "symbian_auto_ptr.h"
#include "cl_settings.h"
#include "context_log.hrh"

CAknSettingItem* CCLSettingList::CreateSettingItemL( TInt identifier )
{
	CALLSTACKITEM(_L("CCLSettingList::CreateSettingItemL"));

	CAknSettingItem* settingItem = NULL;
	
	switch(identifier) {
	case SETTING_JABBER_NICK:
		settingItem = jabbernick = new (ELeave) 
			CAknTextSettingItem(identifier, iJabberNick );
		break;
	case SETTING_JABBER_PASS:
		settingItem = jabberpass = new (ELeave) 
			CAknTextSettingItem(identifier, iJabberPass );
		break;
	case SETTING_UPLOAD_URLBASE:
		iUrlBase=DEFAULT_UPLOAD_URLBASE;
		settingItem = urlbase = new (ELeave) 
			CAknTextSettingItem(identifier, iUrlBase );
		break;
	case SETTING_WAP_AP:
		iWapIap=1;
		settingItem = wap_list = new (ELeave) 
			CAknEnumeratedTextPopupSettingItem(identifier, iWapIap );
		break;
	case SETTING_PRESENCE_ENABLE:
		iPresence=1;
		settingItem = presence_list = new (ELeave) 
			CAknEnumeratedTextPopupSettingItem(identifier, iPresence );
		break;
	case SETTING_OPTIONS_ENABLE:
		iOptions=0;
		settingItem = options_list = new (ELeave) 
			CAknEnumeratedTextPopupSettingItem(identifier, iOptions );
		break;
	case SETTING_PICTURE_ENABLE:
		iPictures=0;
		settingItem = picture_list = new (ELeave) 
			CAknEnumeratedTextPopupSettingItem(identifier, iPictures );
		break;
	case SETTING_LOGGING_ENABLE:
		iLogging=1;
		settingItem = logging_list = new (ELeave) 
			CAknEnumeratedTextPopupSettingItem(identifier, iLogging );
		break;
	case SETTING_DELETE_UPLOADED:
		iDelUploaded=1;
		settingItem = del_list = new (ELeave) 
			CAknEnumeratedTextPopupSettingItem(identifier, iDelUploaded );
		break;
	case SETTING_IP_AP:
		iIpIap=1;
		settingItem = ip_list = new (ELeave) 
			CAknEnumeratedTextPopupSettingItem(identifier, iIpIap );
		break;
	}
	return settingItem;
}

void CCLSettingList::SetListData()
{
	CALLSTACKITEM(_L("CCLSettingList::SetListData"));

	CAknEnumeratedTextPopupSettingItem *list=wap_list;
	TInt *list_id=&iWapIap;
	for (int i=0; i<2; i++) {
		auto_ptr<CCommsDatabase> db(CCommsDatabase::NewL(EDatabaseTypeIAP));
		CCommsDbTableView* iViewP=db->OpenTableLC(TPtrC(IAP));
		CleanupStack::Pop();
		auto_ptr<CCommsDbTableView> iView(iViewP);

		auto_ptr<CArrayPtr< CAknEnumeratedText > > enumarr(
			new (ELeave) CArrayPtrFlat< CAknEnumeratedText >(4));
		auto_ptr<CArrayPtr< HBufC > > popuparr( 
			new (ELeave) CArrayPtrFlat< HBufC >(4));

		TInt err;
		TUint32 id=0;
		bool seen=false;
		while( (err=iView->GotoNextRecord()) == KErrNone ) {
			iView->ReadUintL(TPtrC(COMMDB_ID), id);
			if ( (TInt)id==*list_id) { seen=true; }
			auto_ptr<HBufC> text(HBufC::NewL(KCommsDbSvrMaxColumnNameLength));
			TPtr ptr(text->Des());
			iView->ReadTextL(TPtrC(COMMDB_NAME), ptr);
			auto_ptr<HBufC> popup(text->AllocL());
			auto_ptr<CAknEnumeratedText> item(new (ELeave) CAknEnumeratedText(TInt(id), text.get()));
			text.release();
			enumarr->AppendL(item.get());
			item.release();
			popuparr->AppendL(popup.get());
			popup.release();
		}
		if (!seen) *list_id=id;

		//FIXME: what if no records?
		list->SetEnumeratedTextArrays(enumarr.release(), popuparr.release());
		list->HandleTextArrayUpdateL();
		list->UpdateListBoxTextL ();
		list=ip_list;
		list_id=&iIpIap;
	}
	presence_list->UpdateListBoxTextL();
}

void CCLSettingList::UpdateSettings()
{
	CALLSTACKITEM(_L("CCLSettingList::UpdateSettings"));

	jabbernick->LoadL();
	jabbernick->UpdateListBoxTextL ();

	presence_list->LoadL();
	presence_list->UpdateListBoxTextL ();

	options_list->LoadL();
	options_list->UpdateListBoxTextL ();

	picture_list->LoadL();
	picture_list->UpdateListBoxTextL ();

	jabberpass->LoadL();
	jabberpass->UpdateListBoxTextL ();

	urlbase->LoadL();
	urlbase->UpdateListBoxTextL ();
}

// ================= MEMBER FUNCTIONS =======================

CSettingsView* CSettingsView::NewL(MApp_context& Context)
{
	CALLSTACKITEM(_L("CSettingsView::NewL"));

	auto_ptr<CSettingsView> ret(new (ELeave) CSettingsView(Context));
	ret->ConstructL();
	return ret.release();
}

CSettingsView::CSettingsView(MApp_context& Context) : MContextBase(Context)
{
	CALLSTACKITEM(_L("CSettingsView::CSettingsView"));

}

// ---------------------------------------------------------
// CSettingsView::ConstructL(const TRect& aRect)
// EPOC two-phased constructor
// ---------------------------------------------------------
//
void CSettingsView::ConstructL()
{
	CALLSTACKITEM(_L("CSettingsView::ConstructL"));

	BaseConstructL( R_SETTINGS_VIEW );
}


// ---------------------------------------------------------
// CSettingsView::~CSettingsView()
// ?implementation_description
// ---------------------------------------------------------
//
CSettingsView::~CSettingsView()
{
	CALLSTACKITEM(_L("CSettingsView::~CSettingsView"));

	if ( iListBox )
        {
		AppUi()->RemoveFromViewStack( *this, iListBox );
        }
	
	delete iListBox;
}

// ---------------------------------------------------------
// TUid CSettingsView::Id()
// ?implementation_description
// ---------------------------------------------------------
//
TUid CSettingsView::Id() const
{
	CALLSTACKITEM(_L("CSettingsView::Id"));

	return KSettingsViewId;
}

// ---------------------------------------------------------
// CSettingsView::HandleCommandL(TInt aCommand)
// ?implementation_description
// ---------------------------------------------------------
//
void CSettingsView::HandleCommandL(TInt aCommand)
{   
	CALLSTACKITEM(_L("CSettingsView::HandleCommandL"));

	switch ( aCommand )
        {
        case EAknSoftkeyOk:
	case Econtext_logCmdSettingsSave:
		{
			iListBox->StoreSettingsL();
			TBuf<50> prev_jabber, prev_jabberpass, prev_urlbase;
			TInt prev;
			if (!Settings().GetSettingL(SETTING_WAP_AP, prev) || prev!=iListBox->iWapIap)
				Settings().WriteSettingL(SETTING_WAP_AP, iListBox->iWapIap);
			if (!Settings().GetSettingL(SETTING_IP_AP, prev) || prev!=iListBox->iIpIap)
				Settings().WriteSettingL(SETTING_IP_AP, iListBox->iIpIap);
			if (!Settings().GetSettingL(SETTING_PRESENCE_ENABLE, prev) || prev!=iListBox->iPresence)
				Settings().WriteSettingL(SETTING_PRESENCE_ENABLE, iListBox->iPresence);
			if (!Settings().GetSettingL(SETTING_OPTIONS_ENABLE, prev) || prev!=iListBox->iOptions)
				Settings().WriteSettingL(SETTING_OPTIONS_ENABLE, iListBox->iOptions);
			if (!Settings().GetSettingL(SETTING_PICTURE_ENABLE, prev) || prev!=iListBox->iPictures)
				Settings().WriteSettingL(SETTING_PICTURE_ENABLE, iListBox->iPictures);
			if (!Settings().GetSettingL(SETTING_LOGGING_ENABLE, prev) || prev!=iListBox->iLogging)
				Settings().WriteSettingL(SETTING_LOGGING_ENABLE, iListBox->iLogging);
			if (!Settings().GetSettingL(SETTING_DELETE_UPLOADED, prev) || prev!=iListBox->iDelUploaded)
				Settings().WriteSettingL(SETTING_DELETE_UPLOADED, iListBox->iDelUploaded);

			if (Settings().GetSettingL(SETTING_JABBER_NICK, prev_jabber)) {
				if (iListBox->iJabberNick.Compare(prev_jabber)) {
					Settings().WriteSettingL(SETTING_JABBER_NICK, iListBox->iJabberNick);
				}
			} else if (iListBox->iJabberNick.Length()>0) {
				Settings().WriteSettingL(SETTING_JABBER_NICK, iListBox->iJabberNick);
			}

			if (Settings().GetSettingL(SETTING_JABBER_PASS, prev_jabberpass)) {
				if (iListBox->iJabberPass.Compare(prev_jabberpass)) {
					Settings().WriteSettingL(SETTING_JABBER_PASS, iListBox->iJabberPass);
				}
			} else if (iListBox->iJabberPass.Length()>0) {
				Settings().WriteSettingL(SETTING_JABBER_PASS, iListBox->iJabberPass);
			}

			if (Settings().GetSettingL(SETTING_UPLOAD_URLBASE, prev_urlbase)) {
				if (iListBox->iUrlBase.Compare(prev_urlbase)) {
					Settings().WriteSettingL(SETTING_UPLOAD_URLBASE, iListBox->iUrlBase);
				}
			} else if (iListBox->iUrlBase.Length()>0) {
				Settings().WriteSettingL(SETTING_UPLOAD_URLBASE, iListBox->iUrlBase);
			}
		}
		AppUi()->HandleCommandL( Econtext_logCmdSettingsSave );
		break;
        default:
		AppUi()->HandleCommandL( Econtext_logCmdSettingsCancel );
		break;
        }
}

// ---------------------------------------------------------
// CSettingsView::HandleClientRectChange()
// ---------------------------------------------------------
//
void CSettingsView::HandleClientRectChange()
{
	CALLSTACKITEM(_L("CSettingsView::HandleClientRectChange"));

	if ( iListBox )
        {
		iListBox->SetRect( ClientRect() );
        }
}

// ---------------------------------------------------------
// CSettingsView::DoActivateL(...)
// ?implementation_description
// ---------------------------------------------------------
//
void CSettingsView::DoActivateL(
				const TVwsViewId& /*aPrevViewId*/,TUid /*aCustomMessageId*/,
				const TDesC8& /*aCustomMessage*/)
{
	CALLSTACKITEM(_L("CSettingsView::DoActivateL"));

	if (!iListBox)
        {
		iListBox = new (ELeave) CCLSettingList;
		iListBox->SetMopParent(this);
		iListBox->ConstructFromResourceL(R_CL_SETTINGS_LIST);
		iListBox->iPresence=1;
		iListBox->iLogging=1;
		iListBox->iDelUploaded=1;
		TInt wap, presence, picture, options; TBuf<50> jabbernick, jabberpass, urlbase;
		TInt del_uploaded, logging;
		if (Settings().GetSettingL(SETTING_WAP_AP, wap))
			iListBox->iWapIap=wap;
		if (Settings().GetSettingL(SETTING_IP_AP, wap))
			iListBox->iIpIap=wap;
		if (Settings().GetSettingL(SETTING_PRESENCE_ENABLE, presence))
			iListBox->iPresence=presence;
		if (Settings().GetSettingL(SETTING_OPTIONS_ENABLE, options))
			iListBox->iOptions=options;
		if (Settings().GetSettingL(SETTING_DELETE_UPLOADED, del_uploaded))
			iListBox->iDelUploaded=del_uploaded;
		if (Settings().GetSettingL(SETTING_LOGGING_ENABLE, logging))
			iListBox->iLogging=logging;
		if (Settings().GetSettingL(SETTING_PICTURE_ENABLE, picture))
			iListBox->iPictures=picture;
		if (Settings().GetSettingL(SETTING_JABBER_NICK, jabbernick))  {
			iListBox->iJabberNick=jabbernick;
		}
		if (Settings().GetSettingL(SETTING_JABBER_PASS, jabberpass))  {
			iListBox->iJabberPass=jabberpass;
		}
		if (Settings().GetSettingL(SETTING_UPLOAD_URLBASE, urlbase))  {
			iListBox->iUrlBase=urlbase;
		}
		iListBox->UpdateSettings();

		iListBox->SetListData();
		AppUi()->AddToStackL( *this, iListBox );
		iListBox->MakeVisible(ETrue);
		iListBox->SetRect(ClientRect());
		iListBox->ActivateL();
		iListBox->DrawNow();
		
        } 
}

// ---------------------------------------------------------
// CSettingsView::HandleCommandL(TInt aCommand)
// ?implementation_description
// ---------------------------------------------------------
//
void CSettingsView::DoDeactivate()
{
	CALLSTACKITEM(_L("CSettingsView::DoDeactivate"));

	if ( iListBox )
        {
		AppUi()->RemoveFromViewStack( *this, iListBox );
        }
	
	delete iListBox;
	iListBox = NULL;
}

// End of File

